;;; find-header.el --- provide hypertext style header file referal

;; Copyright (C) 1994 Robert Nicholson
;;
;; Author: Robert Nicholson <robert@steffi.demon.co.uk>
;; Version: 1.1

;;; LCD Archive Entry:
;;; find-header|Robert Nicholson|robert@steffi.demon.co.uk|
;;; Provide hypertext style header file referal.|
;;; 14-May-94|1.1|Contact author

;;; This file is not part of GNU Emacs.

;;; GNU Emacs is free software; you can redistribute it and/or modify
;;; it under the terms of the GNU General Public License as published by
;;; the Free Software Foundation; either version 2, or (at your option)
;;; any later version.

;;; GNU Emacs is distributed in the hope that it will be useful,
;;; but WITHOUT ANY WARRANTY; without even the implied warranty of
;;; MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
;;; GNU General Public License for more details.

;;; You should have received a copy of the GNU General Public License
;;; along with GNU Emacs; see the file COPYING.  If not, write to
;;; the Free Software Foundation, 675 Mass Ave, Cambridge, MA 02139, USA.

;;; Commentary:

;; Purpose:
;;
;; This small package provides facilities to refer to header files "#included"
;; in implementation files:
;;
;;     When point is on a line referring to a header file, locate that file and
;;; find-header.el --- provide hypertext style header file referal

;; Copyright (C) 1994 Robert Nicholson
;;
;; Author: Robert Nicholson <robert@steffi.demon.co.uk>
;; Version: 1.1

;;; LCD Archive Entry:
;;; find-header|Robert Nicholson|robert@steffi.demon.co.uk|
;;; Provide hypertext style header file referal.|
;;; 14-May-94|1.1|Contact author

;;; This file is not part of GNU Emacs.

;;; GNU Emacs is free software; you can redistribute it and/or modify
;;; it under the terms of the GNU General Public License as published by
;;; the Free Software Foundation; either version 2, or (at your option)
;;; any later version.

;;; GNU Emacs is distributed in the hope that it will be useful,
;;; but WITHOUT ANY WARRANTY; without even the implied warranty of
;;; MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
;;; GNU General Public License for more details.

;;; You should have received a copy of the GNU General Public License
;;; along with GNU Emacs; see the file COPYING.  If not, write to
;;; the Free Software Foundation, 675 Mass Ave, Cambridge, MA 02139, USA.

;;; Commentary:

;; Purpose:
;;
;; This small package provides facilities to refer to header files "#included"
;; in implementation files:
;;
;;     When point is on a line referring to a header file, locate that file and
;;     load it either into other-window or other-frame.
;;
;;     The search path is specified through a list of strings in the 
;;     fh-include-path: variable
;;     
;;     ("." "/usr/local/include" "/usr/include")
;;
;; Installation:
;; 
;; To use, autoload in your ~/.emacs the following functions.
;;
;;     (autoload 'fh-open-header-file-other-window "find-header"
;;         "Locate header file and load it into other window" t)
;; 
;;     (autoload 'fh-open-header-file-other-frame "find-header"
;;         "Locate header file and load it into other frame" t)
;;
;; If you use Barry Warsaw's cc-mode create the appropriate bindings.
;;
;;     (add-hook 'c-mode-common-hook '(lambda()
;;		 (local-set-key "\M-o" 'fh-open-header-file-other-window)	 
;;		 (local-set-key "\M-O" 'fh-open-header-file-other-frame)))
;; Code:
 
(defvar fh-include-path '("." "/usr/local/include" "/usr/include")
"List of directories to search for header file") 

(defun fh-current-line ()
  "Return the contents of the current line"
  (save-excursion
    (beginning-of-line)
    (let ((start (point)))
	(end-of-line)
      (buffer-substring start (point)))))

(defun fh-header-file-name ()
  "Return the name of the header file or nil if none"
    (let* ((line (fh-current-line))
	   (dummy (string-match ".* [<\"]+\\(.*\\)[>\"]+" line))
	   (headername (fh-file-in-path
			(substring line (match-beginning 1) (match-end 1))
			fh-include-path)))
      headername))
		
(defun fh-open-header-file-other-window ()
  "Open the header file on the current line in other window"
  (interactive)
  (if (save-excursion
      (beginning-of-line)
      (looking-at ".* [<\"]+\\(.*\\)[>\"]+"))
      (let ((headername (fh-header-file-name)))
	(if headername (find-file-other-window headername)))
    (message "Not on a header file")))

(defun fh-open-corresponding-file ()
  "If on current buffer is header then open implementation and vica versa"
  (interactive)
  (save-excursion
       (let* ((pathname (buffer-file-name)))
;        (string-match ".*/\\([^.]+\\)\.\\(.+\\)" pathname)
		 (string-match "\\(.*\\)\.\\([hm]\\)" pathname)
	  (setq fname  (substring pathname (match-beginning 1) (match-end 1))
                ext    (substring pathname (match-beginning 2) (match-end 2)))
;	  (message fname)
;	  (sit-for 5)
	  (cond
	   ((string= ext "h") (find-file-other-window (concat fname ".m")))
	   ((string= ext "m") (find-file-other-window (concat fname ".h")))
	   ))))

(defun fh-open-header-file-other-frame ()
  "Open the header file on the current line in other frame"
  (interactive)
  (if (save-excursion
	(beginning-of-line)
	(looking-at ".* [<\"]+\\(.*\\)[>\"]+"))
      (let ((headername (fh-header-file-name)))
	(if headername (find-file-other-frame headername)))
    (message "Not on a header file")))

(defun fh-file-in-path (file pathlist)
  "Return qualified path name for a file if found in pathlist or nil"
  (if pathlist
      (progn
	(setq filename (concat (car pathlist) "/" file))
	(if (file-exists-p filename)
	    filename
	  (fh-file-in-path file (cdr pathlist))))))

		      (provide 'find-header)

;;; find-header.el ends here
